//
//  NSObject+Adulterated.m
//  Pure
//
//  Created by Craig Hockenberry on 5/19/16.
//  Copyright © 2016 Craig Hockenberry. All rights reserved.
//

#import <objc/objc-runtime.h>

#import "NSObject+Adulterated.h"

@implementation NSObject (Adulterated)

+ (void)swizzleInstanceSelector:(SEL)originalSelector withNewSelector:(SEL)newSelector
{
	Method originalMethod = class_getInstanceMethod(self, originalSelector);
	Method newMethod = class_getInstanceMethod(self, newSelector);
	
	BOOL methodAdded = class_addMethod([self class],
									   originalSelector,
									   method_getImplementation(newMethod),
									   method_getTypeEncoding(newMethod));
	
	if (methodAdded) {
		class_replaceMethod([self class],
							newSelector,
							method_getImplementation(originalMethod),
							method_getTypeEncoding(originalMethod));
	}
	else {
		method_exchangeImplementations(originalMethod, newMethod);
	}
}


+ (void)load
{
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		// override respondsToSelector
		{
			SEL originalSelector = @selector(respondsToSelector:);
			SEL overrideSelector = @selector(__loggingRespondsToSelector:);
			
			[self swizzleInstanceSelector:originalSelector withNewSelector:overrideSelector];
		}
		// override methodForSelector
		{
			SEL originalSelector = @selector(methodForSelector:);
			SEL overrideSelector = @selector(__loggingMethodForSelector:);
			
			[self swizzleInstanceSelector:originalSelector withNewSelector:overrideSelector];
		}
	});
}


- (BOOL)__loggingRespondsToSelector:(SEL)aSelector
{
	printf("check for [%s %s]\n", [NSStringFromClass([self class]) UTF8String], [NSStringFromSelector(aSelector) UTF8String]);
	
	// call the original respondsToSelector: implementation
	return [self __loggingRespondsToSelector:aSelector];
}

- (IMP)__loggingMethodForSelector:(SEL)aSelector
{
	printf("lookup for [%s %s]\n", [NSStringFromClass([self class]) UTF8String], [NSStringFromSelector(aSelector) UTF8String]);
	
	// call the original methodForSelector: implementation
	return [self __loggingMethodForSelector:aSelector];
}


@end
